package com.tzld.longarticle.recommend.server.service.filter;

import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.service.ServiceBeanFactory;
import com.tzld.longarticle.recommend.server.service.filter.strategy.BadStrategy;
import com.tzld.longarticle.recommend.server.service.filter.strategy.ColdStartBackupFilterStrategy;
import com.tzld.longarticle.recommend.server.service.filter.strategy.HistoryTitleStrategy;
import com.tzld.longarticle.recommend.server.service.filter.strategy.SensitiveStrategy;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Slf4j
@Service
public class FilterService {
    private final ExecutorService pool = ThreadPoolFactory.filterPool();

    public FilterResult filter(FilterParam param) {
        return contentFilter(param);
    }

    private FilterResult contentFilter(FilterParam param) {
        FilterResult result = new FilterResult();

        List<FilterStrategy> strategies = getStrategies(param);
        log.info("FilterStrategy {}", JSONUtils.toJson(CommonCollectionUtils.toList(strategies,
                s -> s.getClass().getSimpleName())));
        CountDownLatch cdl = new CountDownLatch(strategies.size());
        List<Future<FilterResult>> futures = new ArrayList<>();
        for (final FilterStrategy strategy : strategies) {
            Future<FilterResult> future = pool.submit(() -> {
                try {
                    return strategy.filter(param);
                } finally {
                    cdl.countDown();
                }
            });
            futures.add(future);
        }
        try {
            cdl.await(10000, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            log.error("filter error", e);
            return null;
        }

        List<List<String>> contentIdsList = new ArrayList<>();
        List<Content> filterContents = new ArrayList<>();
        Set<String> filterContentIds = new HashSet<>();
        for (Future<FilterResult> f : futures) {
            try {
                FilterResult filterResult = f.get();
                contentIdsList.add(filterResult.getContentIds());
                if (CollectionUtils.isNotEmpty(filterResult.getFilterContent())) {
                    for (Content content : filterResult.getFilterContent()) {
                        if (!filterContentIds.contains(content.getId())) {
                            filterContentIds.add(content.getId());
                            filterContents.add(content);
                        }
                    }
                }
            } catch (Exception e) {
                log.error("future get error ", e);
            }
        }
        if (CollectionUtils.isEmpty(contentIdsList)) {
            return result;
        }
        List<String> contentIds = param.getContents().stream().map(Content::getId).collect(Collectors.toList());
        for (int i = 0; i < contentIdsList.size(); ++i) {
            contentIds.retainAll(contentIdsList.get(i));
        }
        result.setContentIds(contentIds);
        result.setFilterContent(filterContents);
        return result;
    }

    private List<FilterStrategy> getStrategies(FilterParam param) {
        if (param.isBackup()) {
            List<FilterStrategy> strategies = new ArrayList<>();
            strategies.add(ServiceBeanFactory.getBean(HistoryTitleStrategy.class));
            strategies.add(ServiceBeanFactory.getBean(BadStrategy.class));
            strategies.add(ServiceBeanFactory.getBean(SensitiveStrategy.class));
            strategies.add(ServiceBeanFactory.getBean(ColdStartBackupFilterStrategy.class));
            return strategies;
        }
        List<FilterStrategy> strategies = new ArrayList<>();
        strategies.add(ServiceBeanFactory.getBean(HistoryTitleStrategy.class));
        strategies.add(ServiceBeanFactory.getBean(BadStrategy.class));
        strategies.add(ServiceBeanFactory.getBean(SensitiveStrategy.class));
        return strategies;
    }
}
