package com.tzld.longarticle.recommend.server.service.recall;

import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.model.ContentHisPublishArticle;
import com.tzld.longarticle.recommend.server.remote.AIGCRemoteService;
import com.tzld.longarticle.recommend.server.repository.aigc.CrawlerMetaArticleRepository;
import com.tzld.longarticle.recommend.server.repository.crawler.ArticleRepository;
import com.tzld.longarticle.recommend.server.repository.entity.aigc.CrawlerMetaArticle;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.service.AccountIndexAvgViewCountService;
import com.tzld.longarticle.recommend.server.service.recall.strategy.ColdStartBackupRecallStrategy;
import com.tzld.longarticle.recommend.server.service.recall.strategy.DefaultRecallStrategy;
import com.tzld.longarticle.recommend.server.service.score.AvgReadDTO;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import com.tzld.longarticle.recommend.server.util.Md5Util;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Service
@Slf4j
public class RecallService implements ApplicationContextAware {

    @Autowired
    ArticleRepository articleRepository;
    @Autowired
    CrawlerMetaArticleRepository crawlerMetaArticleRepository;
    @Autowired
    AIGCRemoteService aigcRemoteService;
    @Autowired
    AccountIndexAvgViewCountService accountIndexAvgViewCountService;

    private final Map<String, RecallStrategy> strategyMap = new HashMap<>();
    private ApplicationContext applicationContext;
    private final ExecutorService pool = ThreadPoolFactory.recallPool();


    @PostConstruct
    public void init() {
        Map<String, RecallStrategy> type = applicationContext.getBeansOfType(RecallStrategy.class);
        for (Map.Entry<String, RecallStrategy> entry : type.entrySet()) {
            RecallStrategy value = entry.getValue();
            strategyMap.put(value.getClass().getSimpleName(), value);
        }
    }

    public RecallResult recall(RecallParam param) {
        log.info("RecallParam {}", JSONUtils.toJson(param));
        List<RecallStrategy> strategies = getRecallStrategy(param);
        log.info("RecallStrategy {}", JSONUtils.toJson(CommonCollectionUtils.toList(strategies,
                s -> s.getClass().getSimpleName())));
        List<Content> content = getAllContent(param);
        param.setContent(content);
        CountDownLatch cdl = new CountDownLatch(strategies.size());
        List<Future<RecallResult.RecallData>> recallResultFutures = new ArrayList<>();
        for (final RecallStrategy strategy : strategies) {
            Future<RecallResult.RecallData> future = pool.submit(() -> {
                try {
                    return strategy.recall(param);
                } finally {
                    cdl.countDown();
                }
            });
            recallResultFutures.add(future);
        }
        try {
            cdl.await(10000, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            log.error("recall error", e);
            return null;
        }

        List<RecallResult.RecallData> results = new ArrayList<>();
        for (Future<RecallResult.RecallData> f : recallResultFutures) {
            try {
                RecallResult.RecallData data = f.get();
                results.add(data);
            } catch (Exception e) {
                log.error("future get error ", e);
            }
        }

        return new RecallResult(results);
    }

    private List<RecallStrategy> getRecallStrategy(RecallParam param) {
        List<RecallStrategy> strategies = new ArrayList<>();
        strategies.add(strategyMap.get(DefaultRecallStrategy.class.getSimpleName()));
        strategies.add(strategyMap.get(ColdStartBackupRecallStrategy.class.getSimpleName()));
        return strategies;
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }

    private List<Content> getAllContent(RecallParam param) {
        List<Content> content = aigcRemoteService.getAllContent(param);
        // 标题历史均值
        setTitleAvgViewCount(content);
        // category 查询
        setContentCategory(content);
        return content;
    }

    public void setContentCategory(List<Content> contentList) {
        long start = System.currentTimeMillis();
        Map<String, String> articleMd5Map = new HashMap<>();
        List<String> md5List = new ArrayList<>();
        for (Content content : contentList) {
            String md5 = generateArticleUniqueMd5(content.getCrawlerLink());
            md5List.add(md5);
            articleMd5Map.put(content.getId(), md5);
        }
        List<CrawlerMetaArticle> categoryList = getByUniqueIndexIn(md5List);
        if (CollectionUtils.isEmpty(categoryList)) {
            return;
        }
        Map<String, String> categoryMap = categoryList.stream().collect(
                Collectors.toMap(CrawlerMetaArticle::getUniqueIndex, CrawlerMetaArticle::getCategory));
        for (Content content : contentList) {
            String md5 = articleMd5Map.get(content.getId());
            content.setCategory(categoryMap.get(md5));
        }
        log.info("setContentCategory cost:{}", System.currentTimeMillis() - start);
    }

    private List<CrawlerMetaArticle> getByUniqueIndexIn(List<String> md5List) {
        if (CollectionUtils.isEmpty(md5List)) {
            return new ArrayList<>();
        }
        return crawlerMetaArticleRepository.getByUniqueIndexIn(md5List);
    }


    public static String generateArticleUniqueMd5(String url) {
        // Extract parts from the URL
        String biz = extractParameter(url, "biz=");
        String idx = extractParameter(url, "&idx=");
        String sn = extractParameter(url, "&sn=");

        // Combine the parts and encode them to bytes
        String urlBit = String.format("%s-%s-%s", biz, idx, sn);
        return Md5Util.encoderByMd5(urlBit);
    }

    private static String extractParameter(String url, String parameter) {
        int start = url.indexOf(parameter) + parameter.length();
        if (start == -1 || start == url.length()) {
            return "";
        }
        int end = url.indexOf("&", start);
        if (end == -1) {
            return url.substring(start);
        } else {
            return url.substring(start, end);
        }
    }

    public void setTitleAvgViewCount(List<Content> contentList) {
        long start = System.currentTimeMillis();

        List<String> titleList = contentList.stream().map(Content::getTitle).collect(Collectors.toList());
        List<String> crawlerTitleList = contentList.stream().map(Content::getCrawlerTitle).collect(Collectors.toList());
        titleList.addAll(crawlerTitleList);
        List<Article> hisArticleList = articleRepository.getByTitleIn(titleList);
        Map<String, Map<Integer, List<Article>>> map = hisArticleList.stream()
                .collect(Collectors.groupingBy(Article::getTitle, Collectors.groupingBy(Article::getItemIndex)));
        for (Content content : contentList) {
            List<Article> hisArticles = new ArrayList<>();
            Map<Integer, List<Article>> indexArticleMap = map.get(content.getTitle());
            if (Objects.isNull(indexArticleMap)) {
                indexArticleMap = map.get(content.getCrawlerTitle());
            } else if (!content.getTitle().equals(content.getCrawlerTitle())) {
                Map<Integer, List<Article>> crawlerTitleIndexArticleMap = map.get(content.getCrawlerTitle());
                if (Objects.nonNull(crawlerTitleIndexArticleMap)) {
                    for (Map.Entry<Integer, List<Article>> entry : crawlerTitleIndexArticleMap.entrySet()) {
                        if (indexArticleMap.containsKey(entry.getKey())) {
                            List<Article> articleList = indexArticleMap.get(entry.getKey());
                            articleList.addAll(entry.getValue());
                            indexArticleMap.put(entry.getKey(), articleList);
                        } else {
                            indexArticleMap.put(entry.getKey(), entry.getValue());
                        }
                    }
                }
            }
            if (Objects.nonNull(indexArticleMap) && !indexArticleMap.isEmpty()) {
                for (Map.Entry<Integer, List<Article>> indexArticleEntry : indexArticleMap.entrySet()) {
                    List<Article> indexArticleList = indexArticleEntry.getValue();
                    hisArticles.addAll(indexArticleList);
                }
            }
            content.setHisPublishArticleList(new ArrayList<>());
            for (Article hisArticle : hisArticles) {
                ContentHisPublishArticle article = new ContentHisPublishArticle();
                BeanUtils.copyProperties(hisArticle, article);
                AvgReadDTO dto = accountIndexAvgViewCountService.getAvgReadDto(hisArticle.getGhId() + "_" + hisArticle.getItemIndex());
                int avgViewCount = 0;
                if (Objects.nonNull(dto)) {
                    article.setInnerAccount(true);
                    avgViewCount = (int) dto.getReadAvg();
                }
                article.setAvgViewCount(avgViewCount);
                if (Objects.nonNull(article.getAvgViewCount()) && article.getAvgViewCount() > 0) {
                    article.setViewCountRate((article.getShowViewCount() * 1.0) / article.getAvgViewCount());
                }
                content.getHisPublishArticleList().add(article);
            }
        }
        log.info("setTitleAvgViewCount cost:{}", System.currentTimeMillis() - start);
    }

    public static void main(String[] args) {
        String url = "http://mp.weixin.qq.com/s?__biz=Mzg2ODk4MTg3OQ==&mid=2247488306&idx=1&sn=93ebadc5bc7161a0dee48355013d3bc4&chksm=cfb6c1cb2bcdd80dd16d5d604d741a0019ae791125265a042d26100ba21ddb9e5c643ecc2264&scene=126&sessionid=1679649075#rd";
        String md5 = generateArticleUniqueMd5(url);
        System.out.println("Generated md5: " + md5);
    }

}
