package com.tzld.longarticle.recommend.server.service.score;


import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.common.enums.RankStrategyEnum;
import com.tzld.longarticle.recommend.server.service.score.strategy.*;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.stereotype.Service;

import javax.annotation.PostConstruct;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

/**
 * @author dyp
 */
@Service
@Slf4j
public class ScoreService implements ApplicationContextAware {

    private final Map<String, ScoreStrategy> strategyMap = new HashMap<>();
    private ApplicationContext applicationContext;
    private final ExecutorService pool = ThreadPoolFactory.scorePool();

    @PostConstruct
    public void init() {
        Map<String, ScoreStrategy> type = applicationContext.getBeansOfType(ScoreStrategy.class);
        for (Map.Entry<String, ScoreStrategy> entry : type.entrySet()) {
            ScoreStrategy value = entry.getValue();
            strategyMap.put(value.getClass().getSimpleName(), value);
        }
    }

    public ScoreResult score(ScoreParam param) {
        log.info("ScoreParam {}", JSONUtils.toJson(param));
        List<ScoreStrategy> strategies = getScoreStrategy(param);
        log.info("ScoreStrategy {}", JSONUtils.toJson(CommonCollectionUtils.toList(strategies,
                s -> s.getClass().getSimpleName())));
        CountDownLatch cdl = new CountDownLatch(strategies.size());
        List<Future<List<Score>>> futures = new ArrayList<>();
        for (final ScoreStrategy strategy : strategies) {
            Future<List<Score>> future = pool.submit(() -> {
                List<Score> result = strategy.score(param);
                cdl.countDown();
                return result;
            });
            futures.add(future);
        }
        try {
            cdl.await(30000, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            log.error("rank error", e);
            return null;
        }

        Map<String, Map<String, Double>> scoreMap = new HashMap<>();
        for (Future<List<Score>> f : futures) {
            try {
                List<Score> data = f.get();
                for (Score score : data) {
                    Map<String, Double> map
                            = scoreMap.computeIfAbsent(score.getContentId(), k -> new HashMap<>());
                    map.put(score.getStrategy(), score.getScore());
                }
            } catch (Exception e) {
                log.error("future get error ", e);
            }
        }

        return new ScoreResult(scoreMap);
    }

    private List<ScoreStrategy> getScoreStrategy(ScoreParam param) {
        List<ScoreStrategy> strategies = new ArrayList<>();
        strategies.add(strategyMap.get(SimilarityStrategy.class.getSimpleName()));
        strategies.add(strategyMap.get(ViewCountStrategy.class.getSimpleName()));
        if (StringUtils.equals(param.getStrategy(), RankStrategyEnum.ArticleRankV3.getStrategy())) {
            strategies.add(strategyMap.get(ViewMultiplierStrategy.class.getSimpleName()));
            strategies.add(strategyMap.get(CategoryStrategy.class.getSimpleName()));
            strategies.add(strategyMap.get(AccountPreDistributeStrategy.class.getSimpleName()));
            strategies.add(strategyMap.get(FlowCtlDecreaseStrategy.class.getSimpleName()));
            strategies.add(strategyMap.get(ViewCountRateStrategy.class.getSimpleName()));
        }
        if (StringUtils.equals(param.getStrategy(), RankStrategyEnum.ArticleRankV4.getStrategy())) {
            strategies.add(strategyMap.get(CategoryStrategy.class.getSimpleName()));
            strategies.add(strategyMap.get(AccountPreDistributeStrategy.class.getSimpleName()));
            strategies.add(strategyMap.get(FlowCtlDecreaseStrategy.class.getSimpleName()));
            strategies.add(strategyMap.get(ViewCountRateStrategy.class.getSimpleName()));
        }

        return strategies;
    }

    @Override
    public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
        this.applicationContext = applicationContext;
    }

}
